-- Temperature logger via RS232 UART
-- carrier frequency = 40 MHz

-- .000 173 610 sec / packet
-- .000 017 360 sec / bit
--  694 or 695 counts (40 MHz) per bit


-- Magic numbers for 57600/8-N-1:

-- Start   347 == '0'
-- Bit 0  1042
-- Bit 1  1736
-- Bit 2  2431
-- Bit 3  3125
-- Bit 4  3820
-- Bit 5  4514
-- Bit 6  5209
-- Bit 7  5903
-- Stop   6598 == '1'
-- EOF    6945


-- Packet 0:  101-----
-- Packet 1:  ---MMMMM
-- Packet 2:  ---LLLLL
-- ...
-- Packet 25: ---MMMMM
-- Packet 26: ---LLLLL
-- Packet 27: 010-----

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity TLOG is
    port (
		CLK:  in std_logic;
		RXD:  in std_logic;
		T0:  out std_logic_vector(7 downto 0);
		T1:  out std_logic_vector(7 downto 0);
		T2:  out std_logic_vector(7 downto 0);
		T3:  out std_logic_vector(7 downto 0);
		T4:  out std_logic_vector(7 downto 0);
		T5:  out std_logic_vector(7 downto 0);
		T6:  out std_logic_vector(7 downto 0);
		T7:  out std_logic_vector(7 downto 0);
		T8:  out std_logic_vector(7 downto 0);
		T9:  out std_logic_vector(7 downto 0);
		T10: out std_logic_vector(7 downto 0);
		T11: out std_logic_vector(7 downto 0);
		T12: out std_logic_vector(7 downto 0);
		T13: out std_logic_vector(7 downto 0);
		T14: out std_logic_vector(7 downto 0);
		T15: out std_logic_vector(7 downto 0);
		T16: out std_logic_vector(7 downto 0);
		T17: out std_logic_vector(7 downto 0);
		T18: out std_logic_vector(7 downto 0);
		T19: out std_logic_vector(7 downto 0);
		T20: out std_logic_vector(7 downto 0);
		T21: out std_logic_vector(7 downto 0);
		T22: out std_logic_vector(7 downto 0);
		T23: out std_logic_vector(7 downto 0);
		T24: out std_logic_vector(7 downto 0);
		T25: out std_logic_vector(7 downto 0)
		);
end TLOG;

architecture Behavioral of TLOG is

-- signals
--------------------------------------------
	signal RX_CYC: std_logic;
	signal RX_CNT: natural range 0 to 8191;
	signal RX_TERM: std_logic;

	signal RX_REG: std_logic_vector(7 downto 0);

	signal NEW_PAC: std_logic;
	signal PAC_CNT: natural range 0 to 31;

begin

-- RX cycle
--------------------------------------------
	process(RXD, RX_TERM)
	begin
		if (RX_TERM = '1') then
			RX_CYC <= '0';
		elsif (RXD'event and RXD = '0') then
			RX_CYC <= '1';
		end if;
	end process;
--------------------------------------------

-- Counter
--------------------------------------------
	process(CLK, RX_CYC)
	begin
		if (CLK'event and CLK = '1') then
			if (RX_CYC = '1') then
				RX_CNT <= RX_CNT + 1;
			else
				RX_CNT <= 0;
			end if;
		end if;
	end process;
--------------------------------------------

-- Terminate cycle
--------------------------------------------
	process(CLK, RX_CNT)
	begin
		if (CLK'event and CLK = '0') then
			if (RX_CNT = 6945) then
				RX_TERM <= '1';
			else
				RX_TERM <= '0';
			end if;
		end if;
	end process;
--------------------------------------------

-- Latch input data
--------------------------------------------
	process(CLK, RX_CNT, RXD)
	begin
		if (CLK'event and CLK = '0') then
			case RX_CNT is
				when 1042 =>
					RX_REG(0) <= RXD;
				when 1736 =>
					RX_REG(1) <= RXD;
				when 2431 =>
					RX_REG(2) <= RXD;
				when 3125 =>
					RX_REG(3) <= RXD;
				when 3820 =>
					RX_REG(4) <= RXD;
				when 4514 =>
					RX_REG(5) <= RXD;
				when 5209 =>
					RX_REG(6) <= RXD;
				when 5903 =>
					RX_REG(7) <= RXD;
				when others =>
					null;
			end case;
		end if;
	end process;
--------------------------------------------

-- Package enclosed by "A0" and "40"
-- delimiters
--------------------------------------------
	process(RX_CYC, RX_CNT, RX_REG)
	begin
		if (RX_CYC'event and RX_CYC = '0') then
			if (RX_REG = "10100000") then
				NEW_PAC <= '1';
			elsif (RX_REG = "01000000") then
				NEW_PAC <= '0';
			else
				null;
			end if;
		end if;				
	end process;
--------------------------------------------

-- Increment packet counter
--------------------------------------------
	process(RX_CYC, NEW_PAC)
	begin
		if (RX_CYC'event and RX_CYC = '1') then
			if (NEW_PAC = '1') then
				PAC_CNT <= PAC_CNT + 1;
			else
				PAC_CNT <= 0;
			end if;
		end if;
	end process;
--------------------------------------------

-- Latch packet data
--------------------------------------------
	process(RX_CYC, PAC_CNT, RX_REG)
	begin
		if (RX_CYC'event and RX_CYC = '0') then
			case PAC_CNT is
				when 1 =>
					T0  <= RX_REG;
				when 2 =>
					T1  <= RX_REG;
				when 3 =>
					T2  <= RX_REG;
				when 4 =>
					T3  <= RX_REG;
				when 5 =>
					T4  <= RX_REG;
				when 6 =>
					T5  <= RX_REG;
				when 7 =>
					T6  <= RX_REG;
				when 8 =>
					T7  <= RX_REG;
				when 9 =>
					T8  <= RX_REG;
				when 10 =>
					T9  <= RX_REG;
				when 11 =>
					T10 <= RX_REG;
				when 12 =>
					T11 <= RX_REG;
				when 13 =>
					T12 <= RX_REG;
				when 14 =>
					T13 <= RX_REG;
				when 15 =>
					T14 <= RX_REG;
				when 16 =>
					T15 <= RX_REG;
				when 17 =>
					T16 <= RX_REG;
				when 18 =>
					T17 <= RX_REG;
				when 19 =>
					T18 <= RX_REG;
				when 20 =>
					T19 <= RX_REG;
				when 21 =>
					T20 <= RX_REG;
				when 22 =>
					T21 <= RX_REG;
				when 23 =>
					T22 <= RX_REG;
				when 24 =>
					T23 <= RX_REG;
				when 25 =>
					T24 <= RX_REG;
				when 26 =>
					T25 <= RX_REG;
				when others =>
					null;
			end case;
		end if;
	end process;
--------------------------------------------

end Behavioral;
